@@ -12,8 +12,9 @@ import ray.dashboard.consts as dashboard_consts
 import ray.dashboard.optional_utils as dashboard_optional_utils
 import ray.dashboard.utils as dashboard_utils
 from ray import ActorID, NodeID
-from ray._private.metrics_agent import PrometheusServiceDiscoveryWriter
 from ray._common.network_utils import build_address
+from ray._common.usage.usage_constants import CLUSTER_METADATA_KEY
+from ray._private.metrics_agent import PrometheusServiceDiscoveryWriter
 from ray._private.ray_constants import (
     DEBUG_AUTOSCALING_ERROR,
     DEBUG_AUTOSCALING_STATUS,
@@ -23,7 +24,6 @@ from ray._private.ray_constants import (
     KV_NAMESPACE_DASHBOARD,
     env_integer,
 )
-from ray._common.usage.usage_constants import CLUSTER_METADATA_KEY
 from ray._private.utils import init_grpc_channel
 from ray.autoscaler._private.commands import debug_status
 from ray.core.generated import reporter_pb2, reporter_pb2_grpc
@@ -413,21 +413,32 @@ class ReportHead(SubprocessModule):
 
         Params:
             pid: Required. The PID of the worker.
-            ip: Required. The IP address of the node.
+            ip or node_id: Required. The IP address or hex ID of the node.
 
         """
         pid = req.query.get("pid")
         ip = req.query.get("ip")
+        node_id_hex = req.query.get("node_id")
         if not pid:
             raise ValueError("pid is required")
-        if not ip:
-            raise ValueError("ip is required")
+        if not node_id_hex and not ip:
+            raise ValueError("ip or node_id is required")
 
-        addrs = await self._get_stub_address_by_ip(ip)
-        if not addrs:
-            raise aiohttp.web.HTTPInternalServerError(
-                text=f"Failed to get agent address for node at IP {ip}"
+        if node_id_hex:
+            addrs = await self._get_stub_address_by_node_id(
+                NodeID.from_hex(node_id_hex)
             )
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at node_id {node_id_hex}"
+                )
+        else:
+            addrs = await self._get_stub_address_by_ip(ip)
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at IP {ip}"
+                )
+
         node_id, ip, http_port, grpc_port = addrs
         reporter_stub = self._make_stub(build_address(ip, grpc_port))
         # Default not using `--native` for profiling
@@ -451,29 +462,40 @@ class ReportHead(SubprocessModule):
 
         Params:
             pid: Required. The PID of the worker.
-            ip: Required. The IP address of the node.
+            ip or node_id: Required. The IP address or hex ID of the node.
             duration: Optional. Duration in seconds for profiling (default: 5, max: 60).
             format: Optional. Output format (default: "flamegraph").
             native: Optional. Whether to use native profiling (default: false).
 
         Raises:
             ValueError: If pid is not provided.
-            ValueError: If ip is not provided.
+            ValueError: If ip or node_id is not provided.
             ValueError: If duration exceeds 60 seconds.
             aiohttp.web.HTTPInternalServerError: If there is an internal server error during the profile retrieval.
         """
         pid = req.query.get("pid")
         ip = req.query.get("ip")
+        node_id_hex = req.query.get("node_id")
         if not pid:
             raise ValueError("pid is required")
-        if not ip:
-            raise ValueError("ip is required")
+        if not node_id_hex and not ip:
+            raise ValueError("ip or node_id is required")
 
-        addrs = await self._get_stub_address_by_ip(ip)
-        if not addrs:
-            raise aiohttp.web.HTTPInternalServerError(
-                text=f"Failed to get agent address for node at IP {ip}"
+        if node_id_hex:
+            addrs = await self._get_stub_address_by_node_id(
+                NodeID.from_hex(node_id_hex)
             )
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at node_id {node_id_hex}"
+                )
+        else:
+            addrs = await self._get_stub_address_by_ip(ip)
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at IP {ip}"
+                )
+
         node_id, ip, http_port, grpc_port = addrs
         reporter_stub = self._make_stub(build_address(ip, grpc_port))
 
@@ -517,7 +539,7 @@ class ReportHead(SubprocessModule):
         Params:
             req: A request with the following query parameters:
                 pid: Required. The PID of the GPU training worker.
-                ip: Required. The IP address of the node where the GPU training worker is running.
+                ip or node_id: Required. The IP address or hex ID of the node where the GPU training worker is running.
                 num_iterations: Number of training steps for profiling. Defaults to 4
                     This is the number of calls to the torch Optimizer.step().
 
@@ -536,16 +558,27 @@ class ReportHead(SubprocessModule):
 
         pid = req.query.get("pid")
         ip = req.query.get("ip")
+        node_id_hex = req.query.get("node_id")
         if not pid:
             raise ValueError("pid is required")
-        if not ip:
-            raise ValueError("ip is required")
+        if not node_id_hex and not ip:
+            raise ValueError("ip or node_id is required")
 
-        addrs = await self._get_stub_address_by_ip(ip)
-        if not addrs:
-            raise aiohttp.web.HTTPInternalServerError(
-                text=f"Failed to get agent address for node at IP {ip}, pid {pid}"
+        if node_id_hex:
+            addrs = await self._get_stub_address_by_node_id(
+                NodeID.from_hex(node_id_hex)
             )
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at node_id {node_id_hex}, pid {pid}"
+                )
+        else:
+            addrs = await self._get_stub_address_by_ip(ip)
+            if not addrs:
+                raise aiohttp.web.HTTPInternalServerError(
+                    text=f"Failed to get agent address for node at IP {ip}, pid {pid}"
+                )
+
         node_id, ip, http_port, grpc_port = addrs
         reporter_stub = self._make_stub(build_address(ip, grpc_port))
 
@@ -592,7 +625,7 @@ class ReportHead(SubprocessModule):
 
         Params (1):
             pid: The PID of the worker.
-            ip: The IP address of the node.
+            ip or node_id: The IP address or hex ID of the node.
 
         Params (2):
             task_id: The ID of the task.
@@ -601,7 +634,7 @@ class ReportHead(SubprocessModule):
 
         Raises:
             aiohttp.web.HTTPInternalServerError: If no stub
-                found from the given IP value
+                found from the given IP address or hex ID value
             aiohttp.web.HTTPInternalServerError: If the
                 "task_id" parameter exists but either "attempt_number"
                 or "node id" is missing in the request query.
@@ -652,12 +685,27 @@ class ReportHead(SubprocessModule):
         else:
             pid = int(req.query["pid"])
             ip = req.query.get("ip")
-            addrs = await self._get_stub_address_by_ip(ip)
-            if not addrs:
-                return aiohttp.web.HTTPInternalServerError(
-                    text=f"Failed to execute: no agent address found for node IP {ip}"
+            node_id_hex = req.query.get("node_id")
+
+            if not node_id_hex and not ip:
+                raise ValueError("ip or node_id is required")
+
+            if node_id_hex:
+                addrs = await self._get_stub_address_by_node_id(
+                    NodeID.from_hex(node_id_hex)
                 )
-            _, ip, _, grpc_port = addrs
+                if not addrs:
+                    return aiohttp.web.HTTPInternalServerError(
+                        text=f"Failed to execute: no agent address found for node {node_id_hex}"
+                    )
+                _, ip, _, grpc_port = addrs
+            else:
+                addrs = await self._get_stub_address_by_ip(ip)
+                if not addrs:
+                    return aiohttp.web.HTTPInternalServerError(
+                        text=f"Failed to execute: no agent address found for node IP {ip}"
+                    )
+                _, ip, _, grpc_port = addrs
 
         assert pid is not None
         ip_port = build_address(ip, grpc_port)
@@ -785,6 +833,28 @@ class ReportHead(SubprocessModule):
             status_code=status_code, message=message
         )
 
+    @routes.get("/api/prometheus/sd")
+    async def prometheus_service_discovery(self, req) -> aiohttp.web.Response:
+        """
+        Expose Prometheus metrics targets through HTTP Service Discovery.
+        """
+        content = self.service_discovery.get_latest_service_discovery_content()
+        if not isinstance(content, list):
+            error_message = "service discovery error: content is not a list"
+            logger.warning(error_message)
+            return aiohttp.web.json_response(
+                {"error": error_message},
+                status=dashboard_utils.HTTPStatusCode.INTERNAL_ERROR,
+                headers={"Cache-Control": "no-store"},
+            )
+        return aiohttp.web.Response(
+            text=json.dumps(content),
+            content_type="application/json",
+            charset="utf-8",
+            status=dashboard_utils.HTTPStatusCode.OK,
+            headers={"Cache-Control": "no-store"},
+        )
+
     async def _get_stub_address_by_node_id(
         self, node_id: NodeID
     ) -> Optional[Tuple[NodeID, str, int, int]]:
