@@ -2,6 +2,9 @@ from typing import TYPE_CHECKING, List
 
 from .backpressure_policy import BackpressurePolicy
 from .concurrency_cap_backpressure_policy import ConcurrencyCapBackpressurePolicy
+from .downstream_capacity_backpressure_policy import (
+    DownstreamCapacityBackpressurePolicy,
+)
 from .resource_budget_backpressure_policy import ResourceBudgetBackpressurePolicy
 from ray.data.context import DataContext
 
@@ -14,6 +17,7 @@ if TYPE_CHECKING:
 ENABLED_BACKPRESSURE_POLICIES = [
     ConcurrencyCapBackpressurePolicy,
     ResourceBudgetBackpressurePolicy,
+    DownstreamCapacityBackpressurePolicy,
 ]
 ENABLED_BACKPRESSURE_POLICIES_CONFIG_KEY = "backpressure_policies.enabled"
 
@@ -33,6 +37,7 @@ def get_backpressure_policies(
 __all__ = [
     "BackpressurePolicy",
     "ConcurrencyCapBackpressurePolicy",
+    "DownstreamCapacityBackpressurePolicy",
     "ENABLED_BACKPRESSURE_POLICIES_CONFIG_KEY",
     "get_backpressure_policies",
 ]
