@@ -102,11 +102,6 @@ class OpBufferQueue:
             with self._lock:
                 return self._num_per_split[output_split_idx] > 0
 
-    def has_valid_next(self) -> bool:
-        """Whether next RefBundle is available and valid."""
-        with self._lock:
-            return self._queue.has_next()
-
     def append(self, ref: RefBundle):
         """Append a RefBundle to the queue."""
         with self._lock:
@@ -276,9 +271,8 @@ class OpState:
         operator across (external) input queues"""
         return sum(len(q) for q in self.input_queues)
 
-    def has_valid_input_bundle(self) -> bool:
-        """Check if the operator has a valid bundle in its input queue."""
-        return any(queue.has_valid_next() for queue in self.input_queues)
+    def has_pending_bundles(self) -> bool:
+        return self._pending_dispatch_input_bundles_count() > 0
 
     def add_output(self, ref: RefBundle) -> None:
         """Move a bundle produced by the operator to its outqueue."""
@@ -289,6 +283,7 @@ class OpState:
             warn=not self._warned_on_schema_divergence,
             enforce_schemas=self.op.data_context.enforce_schemas,
         )
+
         self._schema = ref.schema
         self._warned_on_schema_divergence |= diverged
 
@@ -306,8 +301,9 @@ class OpState:
         self.op.metrics.num_alive_actors = actor_info.running
         self.op.metrics.num_restarting_actors = actor_info.restarting
         self.op.metrics.num_pending_actors = actor_info.pending
-        self.op.metrics.num_output_queue_blocks = self.output_queue.num_blocks
-        self.op.metrics.num_output_queue_bytes = self.output_queue.memory_usage
+        for next_op in self.op.output_dependencies:
+            next_op.metrics.num_external_inqueue_blocks += len(ref.blocks)
+            next_op.metrics.num_external_inqueue_bytes += ref.size_bytes()
 
     def refresh_progress_bar(self, resource_manager: ResourceManager) -> None:
         """Update the console with the latest operator progress."""
@@ -352,6 +348,8 @@ class OpState:
             ref = inqueue.pop()
             if ref is not None:
                 self.op.add_input(ref, input_index=i)
+                self.op.metrics.num_external_inqueue_bytes -= ref.size_bytes()
+                self.op.metrics.num_external_inqueue_blocks -= len(ref.blocks)
                 return
 
         assert False, "Nothing to dispatch"
@@ -568,8 +566,17 @@ def update_operator_states(topology: Topology) -> None:
     """Update operator states accordingly for newly completed tasks.
     Should be called after `process_completed_tasks()`."""
 
-    # Call inputs_done() on ops where no more inputs are coming.
     for op, op_state in topology.items():
+        # Drain upstream output queue if current operator is execution finished.
+        # This is needed when the limit is reached, and `mark_execution_finished`
+        # is called manually.
+        if op.execution_finished():
+            for idx, dep in enumerate(op.input_dependencies):
+                upstream_state = topology[dep]
+                # Drain upstream output queue
+                upstream_state.output_queue.clear()
+
+        # Call inputs_done() on ops where no more inputs are coming.
         if op_state.inputs_done_called:
             continue
         all_inputs_done = True
@@ -633,11 +640,7 @@ def get_eligible_operators(
         #   - It's not completed
         #   - It can accept at least one input
         #   - Its input queue has a valid bundle
-        if (
-            not op.completed()
-            and op.should_add_input()
-            and state.has_valid_input_bundle()
-        ):
+        if not op.completed() and op.should_add_input() and state.has_pending_bundles():
             if not in_backpressure:
                 op_runnable = True
                 eligible_ops.append(op)
@@ -789,7 +792,7 @@ def dedupe_schemas_with_validation(
         return bundle, diverged
 
     diverged = True
-    if warn:
+    if warn and enforce_schemas:
         logger.warning(
             f"Operator produced a RefBundle with a different schema "
             f"than the previous one. Previous schema: {old_schema}, "
