@@ -1,5 +1,3 @@
-from typing import List
-
 from ray.data._internal.logical.interfaces import LogicalOperator
 
 
@@ -15,6 +13,6 @@ class Count(LogicalOperator):
 
     def __init__(
         self,
-        input_dependencies: List["LogicalOperator"],
+        input_op: LogicalOperator,
     ):
-        super().__init__("Count", input_dependencies)
+        super().__init__("Count", [input_op])
