@@ -70,6 +70,10 @@ DEFAULT_STREAMING_READ_BUFFER_SIZE = 32 * 1024 * 1024
 
 DEFAULT_ENABLE_PANDAS_BLOCK = True
 
+DEFAULT_PANDAS_BLOCK_IGNORE_METADATA = env_bool(
+    "RAY_DATA_PANDAS_BLOCK_IGNORE_METADATA", False
+)
+
 DEFAULT_READ_OP_MIN_NUM_BLOCKS = 200
 
 DEFAULT_ACTOR_PREFETCHER_ENABLED = False
@@ -79,11 +83,11 @@ DEFAULT_USE_PUSH_BASED_SHUFFLE = bool(
 )
 
 DEFAULT_SHUFFLE_STRATEGY = os.environ.get(
-    "RAY_DATA_DEFAULT_SHUFFLE_STRATEGY", ShuffleStrategy.SORT_SHUFFLE_PULL_BASED
+    "RAY_DATA_DEFAULT_SHUFFLE_STRATEGY", ShuffleStrategy.HASH_SHUFFLE
 )
 
 DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS = env_integer(
-    "RAY_DATA_MAX_HASH_SHUFFLE_AGGREGATORS", 64
+    "RAY_DATA_MAX_HASH_SHUFFLE_AGGREGATORS", 128
 )
 
 DEFAULT_SCHEDULING_STRATEGY = "SPREAD"
@@ -140,7 +144,7 @@ DEFAULT_ENABLE_PROGRESS_BAR_NAME_TRUNCATION = env_bool(
     "RAY_DATA_ENABLE_PROGRESS_BAR_NAME_TRUNCATION", True
 )
 
-DEFAULT_ENFORCE_SCHEMAS = env_bool("RAY_DATA_ALLOW_ENFORCE_SCHEMAS", False)
+DEFAULT_ENFORCE_SCHEMAS = env_bool("RAY_DATA_ENFORCE_SCHEMAS", False)
 
 DEFAULT_ENABLE_GET_OBJECT_LOCATIONS_FOR_METRICS = False
 
@@ -232,23 +236,24 @@ DEFAULT_ACTOR_POOL_UTIL_DOWNSCALING_THRESHOLD: float = env_float(
 @DeveloperAPI
 @dataclass
 class AutoscalingConfig:
-    # Actor Pool utilization threshold for upscaling. Once Actor Pool
-    # exceeds this utilization threshold it will start adding new actors.
-    #
-    # NOTE: Actor Pool utilization is defined as ratio of
-    #
-    #   - Number of submitted tasks to
-    #   - Max number of tasks the current set of Actors in the pool could run
-    #     (defined as Ray Actor's `max_concurrency` * `pool.num_running_actors`)
-    #
-    # This utilization value could exceed 100%, when the number of submitted tasks
-    # exceed available concurrency-slots to run them in the current set of actors.
-    #
-    # This is possible when `max_tasks_in_flight_per_actor` (defaults to 2 x
-    # of `max_concurrency`) > Actor's `max_concurrency` and allows to overlap
-    # task execution with the fetching of the blocks for the next task providing
-    # for ability to negotiate a trade-off between autoscaling speed and resource
-    # efficiency (ie making tasks wait instead of immediately triggering execution)
+    """Configuration for autoscaling of Ray Data.
+
+    Args:
+        actor_pool_util_upscaling_threshold: Actor Pool utilization threshold for upscaling.
+            Once Actor Pool exceeds this utilization threshold it will start adding new actors.
+            Actor Pool utilization is defined as ratio of number of submitted tasks to the
+            number of available concurrency-slots to run them in the current set of actors.
+            This utilization value could exceed 100%, when the number of submitted tasks
+            exceed available concurrency-slots to run them in the current set of actors.
+            This is possible when `max_tasks_in_flight_per_actor`
+            (defaults to 2 x of `max_concurrency`) > Actor's `max_concurrency`
+            and allows to overlap task execution with the fetching of the blocks
+            for the next task providing for ability to negotiate a trade-off
+            between autoscaling speed and resource efficiency (i.e.,
+            making tasks wait instead of immediately triggering execution).
+        actor_pool_util_downscaling_threshold: Actor Pool utilization threshold for downscaling.
+    """
+
     actor_pool_util_upscaling_threshold: float = (
         DEFAULT_ACTOR_POOL_UTIL_UPSCALING_THRESHOLD
     )
@@ -359,6 +364,8 @@ class DataContext:
             to use.
         use_ray_tqdm: Whether to enable distributed tqdm.
         enable_progress_bars: Whether to enable progress bars.
+        enable_operator_progress_bars: Whether to enable progress bars for individual
+            operators during execution.
         enable_progress_bar_name_truncation: If True, the name of the progress bar
             (often the operator name) will be truncated if it exceeds
             `ProgressBar.MAX_NAME_LENGTH`. Otherwise, the full operator name is shown.
@@ -374,7 +381,8 @@ class DataContext:
             retry. This follows same format as :ref:`retry_exceptions <task-retries>` in
             Ray Core. Default to `False` to not retry on any errors. Set to `True` to
             retry all errors, or set to a list of errors to retry.
-        enable_op_resource_reservation: Whether to reserve resources for each operator.
+        op_resource_reservation_enabled: Whether to enable resource reservation for
+            operators to prevent resource contention.
         op_resource_reservation_ratio: The ratio of the total resources to reserve for
             each operator.
         max_errored_blocks: Max number of blocks that are allowed to have errors,
@@ -404,10 +412,42 @@ class DataContext:
         retried_io_errors: A list of substrings of error messages that should
             trigger a retry when reading or writing files. This is useful for handling
             transient errors when reading from remote storage systems.
+        default_hash_shuffle_parallelism: Default parallelism level for hash-based
+            shuffle operations if the number of partitions is unspecifed.
+        max_hash_shuffle_aggregators: Maximum number of aggregating actors that can be
+            provisioned for hash-shuffle aggregations.
+        min_hash_shuffle_aggregator_wait_time_in_s: Minimum time to wait for hash
+            shuffle aggregators to become available, in seconds.
+        hash_shuffle_aggregator_health_warning_interval_s: Interval for health warning
+            checks on hash shuffle aggregators, in seconds.
+        max_hash_shuffle_finalization_batch_size: Maximum batch size for concurrent
+            hash-shuffle finalization tasks. If `None`, defaults to
+            `max_hash_shuffle_aggregators`.
+        join_operator_actor_num_cpus_per_partition_override: Override CPU allocation
+            per partition for join operator actors.
+        hash_shuffle_operator_actor_num_cpus_per_partition_override: Override CPU
+            allocation per partition for hash shuffle operator actors.
+        hash_aggregate_operator_actor_num_cpus_per_partition_override: Override CPU
+            allocation per partition for hash aggregate operator actors.
+        use_polars_sort: Whether to use Polars for tabular dataset sorting operations.
         enable_per_node_metrics: Enable per node metrics reporting for Ray Data,
             disabled by default.
+        override_object_store_memory_limit_fraction: Override the fraction of object
+            store memory limit. If `None`, uses Ray's default.
         memory_usage_poll_interval_s: The interval to poll the USS of map tasks. If `None`,
             map tasks won't record memory stats.
+        dataset_logger_id: Optional logger ID for dataset operations. If `None`, uses
+            default logging configuration.
+        issue_detectors_config: Configuration for issue detection and monitoring during
+            dataset operations.
+        downstream_capacity_backpressure_ratio: Ratio for downstream capacity
+            backpressure control. A higher ratio causes backpressure to kick-in
+            later. If `None`, this type of backpressure is disabled.
+        downstream_capacity_backpressure_max_queued_bundles: Maximum number of queued
+            bundles before applying backpressure. If `None`, no limit is applied.
+        enforce_schemas: Whether to enforce schema consistency across dataset operations.
+        pandas_block_ignore_metadata: Whether to ignore pandas metadata when converting
+            between Arrow and pandas formats for better type inference.
     """
 
     # `None` means the block size is infinite.
@@ -435,13 +475,15 @@ class DataContext:
 
     # Default hash-shuffle parallelism level (will be used when not
     # provided explicitly)
-    default_hash_shuffle_parallelism = DEFAULT_MIN_PARALLELISM
+    default_hash_shuffle_parallelism: int = DEFAULT_MIN_PARALLELISM
 
-    # Max number of aggregating actors that could be provisioned
+    # Max number of aggregators (actors) that could be provisioned
     # to perform aggregations on partitions produced during hash-shuffling
     #
-    # When unset defaults to `DataContext.min_parallelism`
-    max_hash_shuffle_aggregators: Optional[int] = DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS
+    # When unset defaults to the smaller of
+    #   - Total # of CPUs available in the cluster * 2
+    #   - DEFAULT_MAX_HASH_SHUFFLE_AGGREGATORS (128 by default)
+    max_hash_shuffle_aggregators: Optional[int] = None
 
     min_hash_shuffle_aggregator_wait_time_in_s: int = (
         DEFAULT_MIN_HASH_SHUFFLE_AGGREGATOR_WAIT_TIME_IN_S
@@ -459,9 +501,11 @@ class DataContext:
     # When unset defaults to `DataContext.max_hash_shuffle_aggregators`
     max_hash_shuffle_finalization_batch_size: Optional[int] = None
 
-    join_operator_actor_num_cpus_per_partition_override: float = None
-    hash_shuffle_operator_actor_num_cpus_per_partition_override: float = None
-    hash_aggregate_operator_actor_num_cpus_per_partition_override: float = None
+    # (Advanced) Following configuration allows to override `num_cpus` allocation for the
+    # Join/Aggregate/Shuffle workers (utilizing hash-shuffle)
+    join_operator_actor_num_cpus_override: float = None
+    hash_shuffle_operator_actor_num_cpus_override: float = None
+    hash_aggregate_operator_actor_num_cpus_override: float = None
 
     scheduling_strategy: SchedulingStrategyT = DEFAULT_SCHEDULING_STRATEGY
     scheduling_strategy_large_args: SchedulingStrategyT = (
@@ -535,8 +579,13 @@ class DataContext:
         default_factory=_issue_detectors_config_factory
     )
 
+    downstream_capacity_backpressure_ratio: float = None
+    downstream_capacity_backpressure_max_queued_bundles: int = None
+
     enforce_schemas: bool = DEFAULT_ENFORCE_SCHEMAS
 
+    pandas_block_ignore_metadata: bool = DEFAULT_PANDAS_BLOCK_IGNORE_METADATA
+
     def __post_init__(self):
         # The additonal ray remote args that should be added to
         # the task-pool-based data tasks.
