@@ -4,7 +4,6 @@ from typing import TYPE_CHECKING, AsyncGenerator, Optional, Tuple, Union
 
 from starlette.datastructures import State
 from starlette.requests import Request
-from transformers.dynamic_module_utils import init_hf_modules
 from vllm.engine.arg_utils import AsyncEngineArgs
 from vllm.entrypoints.openai.cli_args import FrontendArgs
 from vllm.entrypoints.openai.protocol import ErrorResponse as VLLMErrorResponse
@@ -18,7 +17,10 @@ from ray.llm._internal.serve.configs.openai_api_models import (
     CompletionResponse,
     EmbeddingRequest,
     EmbeddingResponse,
+    ErrorInfo,
     ErrorResponse,
+    ScoreRequest,
+    ScoreResponse,
 )
 from ray.llm._internal.serve.configs.server_models import (
     DiskMultiplexConfig,
@@ -43,6 +45,7 @@ if TYPE_CHECKING:
     from vllm.entrypoints.openai.serving_completion import OpenAIServingCompletion
     from vllm.entrypoints.openai.serving_embedding import OpenAIServingEmbedding
     from vllm.entrypoints.openai.serving_models import OpenAIServingModels
+    from vllm.entrypoints.openai.serving_score import ServingScores
 
 vllm = try_import("vllm")
 logger = get_logger(__name__)
@@ -107,10 +110,6 @@ class VLLMEngine(LLMEngine):
         """
         super().__init__(llm_config)
 
-        # Ensure transformers_modules is initialized early in worker processes.
-        # This is critical for models with trust_remote_code=True to avoid pickle errors.
-        init_hf_modules()
-
         self.llm_config = llm_config
 
         if vllm is None:
@@ -134,6 +133,7 @@ class VLLMEngine(LLMEngine):
         self._oai_serving_chat: Optional["OpenAIServingChat"] = None
         self._oai_serving_completion: Optional["OpenAIServingCompletion"] = None
         self._oai_serving_embedding: Optional["OpenAIServingEmbedding"] = None
+        self._oai_serving_scores: Optional["ServingScores"] = None
 
     async def start(self) -> None:
         """Start the vLLM engine.
@@ -189,6 +189,7 @@ class VLLMEngine(LLMEngine):
         self._oai_serving_chat = state.openai_serving_chat
         self._oai_serving_completion = state.openai_serving_completion
         self._oai_serving_embedding = state.openai_serving_embedding
+        self._oai_serving_scores = state.openai_serving_scores
 
         self._validate_openai_serving_models()
         self._validate_engine_client()
@@ -221,6 +222,11 @@ class VLLMEngine(LLMEngine):
             self._oai_serving_embedding, "create_embedding"
         ), "oai_serving_embedding must have a create_embedding attribute"
 
+    def _validate_openai_serving_scores(self):
+        assert hasattr(
+            self._oai_serving_scores, "create_score"
+        ), "oai_serving_scores must have a create_score attribute"
+
     def _validate_engine_client(self):
         assert hasattr(
             self._engine_client, "check_health"
@@ -350,11 +356,13 @@ class VLLMEngine(LLMEngine):
         )
 
         if isinstance(lora_request, VLLMErrorResponse):
-            raise ValueError(f"Failed to load lora model: {lora_request.message}")
+            raise ValueError(f"Failed to load lora model: {lora_request.error.message}")
 
     def _create_raw_request(
         self,
-        request: Union[CompletionRequest, ChatCompletionRequest, EmbeddingRequest],
+        request: Union[
+            CompletionRequest, ChatCompletionRequest, EmbeddingRequest, ScoreRequest
+        ],
         path: str,
     ) -> Request:
         scope = {
@@ -390,7 +398,7 @@ class VLLMEngine(LLMEngine):
                 yield response
         else:
             if isinstance(chat_response, VLLMErrorResponse):
-                yield ErrorResponse(**chat_response.model_dump())
+                yield ErrorResponse(error=ErrorInfo(**chat_response.error.model_dump()))
             else:
                 yield ChatCompletionResponse(**chat_response.model_dump())
 
@@ -419,7 +427,9 @@ class VLLMEngine(LLMEngine):
                 yield response
         else:
             if isinstance(completion_response, VLLMErrorResponse):
-                yield ErrorResponse(**completion_response.model_dump())
+                yield ErrorResponse(
+                    error=ErrorInfo(**completion_response.error.model_dump())
+                )
             else:
                 yield CompletionResponse(**completion_response.model_dump())
 
@@ -438,10 +448,28 @@ class VLLMEngine(LLMEngine):
         )
 
         if isinstance(embedding_response, VLLMErrorResponse):
-            yield ErrorResponse(**embedding_response.model_dump())
+            yield ErrorResponse(
+                error=ErrorInfo(**embedding_response.error.model_dump())
+            )
         else:
             yield EmbeddingResponse(**embedding_response.model_dump())
 
+    async def score(
+        self, request: ScoreRequest
+    ) -> AsyncGenerator[Union[ScoreResponse, ErrorResponse], None]:
+        self._validate_openai_serving_scores()
+
+        raw_request = self._create_raw_request(request, "/score")
+
+        score_response = await self._oai_serving_scores.create_score(
+            request, raw_request=raw_request
+        )
+
+        if isinstance(score_response, VLLMErrorResponse):
+            yield ErrorResponse(**score_response.model_dump())
+        else:
+            yield ScoreResponse(**score_response.model_dump())
+
     async def check_health(self) -> None:
         assert self._engine_client is not None, "engine_client is not initialized"
 
@@ -454,3 +482,11 @@ class VLLMEngine(LLMEngine):
     async def reset_prefix_cache(self) -> None:
         assert self._engine_client is not None, "engine_client is not initialized"
         await self._engine_client.reset_prefix_cache()
+
+    async def start_profile(self) -> None:
+        assert self._engine_client is not None, "engine_client is not initialized"
+        await self._engine_client.start_profile()
+
+    async def stop_profile(self) -> None:
+        assert self._engine_client is not None, "engine_client is not initialized"
+        await self._engine_client.stop_profile()
