@@ -14,7 +14,7 @@ from ray.rllib.utils.annotations import (
     OverrideToImplementCustomLogic,
 )
 from ray.rllib.utils.checkpoints import Checkpointable
-from ray.rllib.utils.deprecation import (
+from ray._common.deprecation import (
     Deprecated,
     DEPRECATED_VALUE,
     deprecation_warning,
@@ -269,6 +269,8 @@ class RLModule(Checkpointable, abc.ABC):
             DefaultPPOTorchRLModule
         )
         from ray.rllib.algorithms.ppo.ppo_catalog import PPOCatalog
+        from ray.rllib.core.rl_module.rl_module import RLModuleSpec
+        from ray.rllib.core.rl_module.default_model_config import DefaultModelConfig
         import gymnasium as gym
         import torch
 
@@ -300,6 +302,12 @@ class RLModule(Checkpointable, abc.ABC):
 
     .. testcode::
 
+        from ray.rllib.algorithms.ppo.torch.ppo_torch_rl_module import (
+            PPOTorchRLModule
+        )
+        from ray.rllib.algorithms.ppo.ppo_catalog import PPOCatalog
+        from ray.rllib.core.rl_module.rl_module import RLModuleSpec
+        from ray.rllib.core.rl_module.default_model_config import DefaultModelConfig
         import gymnasium as gym
         import torch
 
@@ -327,6 +335,12 @@ class RLModule(Checkpointable, abc.ABC):
 
     .. testcode::
 
+        from ray.rllib.algorithms.ppo.torch.ppo_torch_rl_module import (
+            PPOTorchRLModule
+        )
+        from ray.rllib.algorithms.ppo.ppo_catalog import PPOCatalog
+        from ray.rllib.core.rl_module.rl_module import RLModuleSpec
+        from ray.rllib.core.rl_module.default_model_config import DefaultModelConfig
         import gymnasium as gym
         import torch
 
