@@ -3,7 +3,15 @@ import logging
 import os
 import pickle
 import time
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
 import ray
 from ray._common.network_utils import build_address
@@ -13,9 +21,11 @@ from ray.actor import ActorHandle
 from ray.serve._private.application_state import ApplicationStateManager, StatusOverview
 from ray.serve._private.autoscaling_state import AutoscalingStateManager
 from ray.serve._private.common import (
-    DeploymentHandleSource,
+    RUNNING_REQUESTS_KEY,
     DeploymentID,
+    HandleMetricReport,
     NodeId,
+    ReplicaMetricReport,
     RequestProtocol,
     RequestRoutingInfo,
     RunningReplicaInfo,
@@ -25,6 +35,7 @@ from ray.serve._private.config import DeploymentConfig
 from ray.serve._private.constants import (
     CONTROL_LOOP_INTERVAL_S,
     RAY_SERVE_CONTROLLER_CALLBACK_IMPORT_PATH,
+    RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS,
     RECOVERING_LONG_POLL_BROADCAST_TIMEOUT_S,
     SERVE_CONTROLLER_NAME,
     SERVE_DEFAULT_APP_NAME,
@@ -64,6 +75,7 @@ from ray.serve.generated.serve_pb2 import (
     EndpointSet,
 )
 from ray.serve.schema import (
+    APIType,
     ApplicationDetails,
     DeploymentDetails,
     HTTPOptionsSchema,
@@ -259,40 +271,48 @@ class ServeController:
     def get_pid(self) -> int:
         return os.getpid()
 
-    def record_autoscaling_metrics(
-        self, replica_id: str, window_avg: Optional[float], send_timestamp: float
+    def record_autoscaling_metrics_from_replica(
+        self, replica_metric_report: ReplicaMetricReport
     ):
         logger.debug(
-            f"Received metrics from replica {replica_id}: {window_avg} running requests"
+            f"Received metrics from replica {replica_metric_report.replica_id}: {replica_metric_report.aggregated_metrics.get(RUNNING_REQUESTS_KEY)} running requests"
         )
+        latency = time.time() - replica_metric_report.timestamp
+        latency_ms = latency * 1000
+        if latency_ms > RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS:
+            logger.warning(
+                f"Received autoscaling metrics from replica {replica_metric_report.replica_id} with timestamp {replica_metric_report.timestamp} "
+                f"which is {latency_ms}ms ago. "
+                f"This is greater than the warning threshold RPC latency of {RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS}ms. "
+                "This may indicate a performance issue with the controller try increasing the RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS environment variable."
+            )
         self.autoscaling_state_manager.record_request_metrics_for_replica(
-            replica_id, window_avg, send_timestamp
+            replica_metric_report
         )
 
-    def record_handle_metrics(
-        self,
-        deployment_id: str,
-        handle_id: str,
-        actor_id: Optional[str],
-        handle_source: DeploymentHandleSource,
-        queued_requests: float,
-        running_requests: Dict[str, float],
-        send_timestamp: float,
+    def record_autoscaling_metrics_from_handle(
+        self, handle_metric_report: HandleMetricReport
     ):
         logger.debug(
-            f"Received metrics from handle {handle_id} for deployment {deployment_id}: "
-            f"{queued_requests} queued requests and {running_requests} running requests"
+            f"Received metrics from handle {handle_metric_report.handle_id} for deployment {handle_metric_report.deployment_id}: "
+            f"{handle_metric_report.queued_requests} queued requests and {handle_metric_report.aggregated_metrics[RUNNING_REQUESTS_KEY]} running requests"
         )
+        latency = time.time() - handle_metric_report.timestamp
+        latency_ms = latency * 1000
+        if latency_ms > RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS:
+            logger.warning(
+                f"Received autoscaling metrics from handle {handle_metric_report.handle_id} for deployment {handle_metric_report.deployment_id} with timestamp {handle_metric_report.timestamp} "
+                f"which is {latency_ms}ms ago. "
+                f"This is greater than the warning threshold RPC latency of {RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS}ms. "
+                "This may indicate a performance issue with the controller try increasing the RAY_SERVE_RPC_LATENCY_WARNING_THRESHOLD_MS environment variable."
+            )
         self.autoscaling_state_manager.record_request_metrics_for_handle(
-            deployment_id=deployment_id,
-            handle_id=handle_id,
-            actor_id=actor_id,
-            handle_source=handle_source,
-            queued_requests=queued_requests,
-            running_requests=running_requests,
-            send_timestamp=send_timestamp,
+            handle_metric_report
         )
 
+    def _dump_all_autoscaling_metrics_for_testing(self):
+        return self.autoscaling_state_manager.get_all_metrics()
+
     def _dump_autoscaling_metrics_for_testing(self):
         return self.autoscaling_state_manager.get_metrics()
 
@@ -442,8 +462,6 @@ class ServeController:
             dsm_update_start_time = time.time()
             any_recovering = self.deployment_state_manager.update()
 
-            self.deployment_state_manager.save_checkpoint()
-
             self.dsm_update_duration_gauge_s.set(time.time() - dsm_update_start_time)
             if not self.done_recovering_event.is_set() and not any_recovering:
                 self.done_recovering_event.set()
@@ -461,11 +479,6 @@ class ServeController:
             asm_update_start_time = time.time()
             self.application_state_manager.update()
 
-            self.application_state_manager.save_checkpoint()
-            # ApplicationStateManager.update() can also mutate the
-            # DeploymentStateManager so we need to checkpoint that as well
-            self.deployment_state_manager.save_checkpoint()
-
             self.asm_update_duration_gauge_s.set(time.time() - asm_update_start_time)
         except Exception:
             logger.exception("Exception updating application state.")
@@ -659,6 +672,9 @@ class ServeController:
             if SERVE_ROOT_URL_ENV_KEY in os.environ:
                 return os.environ[SERVE_ROOT_URL_ENV_KEY]
             else:
+                # HTTP is disabled
+                if http_config.host is None:
+                    return ""
                 return (
                     f"http://{build_address(http_config.host, http_config.port)}"
                     f"{http_config.root_path}"
@@ -916,12 +932,30 @@ class ServeController:
         """Gets the current list of all deployments' identifiers."""
         return self.deployment_state_manager._deployment_states.keys()
 
-    def get_serve_instance_details(self) -> Dict:
+    def update_deployment_replicas(
+        self, deployment_id: DeploymentID, target_num_replicas: int
+    ) -> None:
+        """Update the target number of replicas for a deployment.
+
+        Args:
+            deployment_id: The deployment to update.
+            target_num_replicas: The new target number of replicas.
+        """
+        self.deployment_state_manager.set_target_num_replicas(
+            deployment_id, target_num_replicas
+        )
+
+    def get_serve_instance_details(self, source: Optional[APIType] = None) -> Dict:
         """Gets details on all applications on the cluster and system-level info.
 
         The information includes application and deployment statuses, config options,
         error messages, etc.
 
+        Args:
+            source: If provided, returns application
+                statuses for applications matching this API type.
+                Defaults to None, which means all applications are returned.
+
         Returns:
             Dict that follows the format of the schema ServeInstanceDetails.
         """
@@ -930,7 +964,7 @@ class ServeController:
         grpc_config = self.get_grpc_config()
         applications = {}
 
-        app_statuses = self.application_state_manager.list_app_statuses()
+        app_statuses = self.application_state_manager.list_app_statuses(source=source)
 
         # If there are no app statuses, there's no point getting the app configs.
         # Moreover, there might be no app statuses because the GCS is down,
@@ -1107,6 +1141,15 @@ class ServeController:
         """
         self.deployment_state_manager.record_request_routing_info(info)
 
+    def _get_replica_ranks_mapping(self, deployment_id: DeploymentID) -> Dict[str, int]:
+        """Get the current rank mapping for all replicas in a deployment.
+        Args:
+            deployment_id: The deployment ID to get ranks for.
+        Returns:
+            Dictionary mapping replica_id to rank.
+        """
+        return self.deployment_state_manager._get_replica_ranks_mapping(deployment_id)
+
     async def graceful_shutdown(self, wait: bool = True):
         """Set the shutting down flag on controller to signal shutdown in
         run_control_loop().
