@@ -714,6 +714,23 @@ async def start_asgi_http_server(
     # has no use to us.
     logging.getLogger("uvicorn.error").level = logging.CRITICAL
 
+    # Configure SSL if certificates are provided
+    ssl_kwargs = {}
+    if http_options.ssl_keyfile and http_options.ssl_certfile:
+        ssl_kwargs = {
+            "ssl_keyfile": http_options.ssl_keyfile,
+            "ssl_certfile": http_options.ssl_certfile,
+        }
+        if http_options.ssl_keyfile_password:
+            ssl_kwargs["ssl_keyfile_password"] = http_options.ssl_keyfile_password
+        if http_options.ssl_ca_certs:
+            ssl_kwargs["ssl_ca_certs"] = http_options.ssl_ca_certs
+
+        logger.info(
+            f"Starting HTTPS server on {http_options.host}:{http_options.port} "
+            f"with SSL certificate: {http_options.ssl_certfile}"
+        )
+
     # NOTE: We have to use lower level uvicorn Config and Server
     # class because we want to run the server as a coroutine. The only
     # alternative is to call uvicorn.run which is blocking.
@@ -730,6 +747,7 @@ async def start_asgi_http_server(
             access_log=False,
             log_level=None,
             log_config=None,
+            **ssl_kwargs,
         )
     )
 
