@@ -5,30 +5,27 @@ import socket
 import sys
 import time
 import traceback
+import urllib
 from concurrent import futures
 from dataclasses import dataclass
 from itertools import chain
-import urllib
 from threading import Event, Lock, RLock, Thread
 from typing import Callable, Dict, List, Optional, Tuple
 
 import grpc
 
-# Import psutil after ray so the packaged version is used.
-import psutil
-
 import ray
 import ray.core.generated.ray_client_pb2 as ray_client_pb2
 import ray.core.generated.ray_client_pb2_grpc as ray_client_pb2_grpc
 import ray.core.generated.runtime_env_agent_pb2 as runtime_env_agent_pb2
+from ray._common.network_utils import build_address, is_localhost
 from ray._private.client_mode_hook import disable_client_hook
-from ray._raylet import GcsClient
 from ray._private.parameter import RayParams
 from ray._private.runtime_env.context import RuntimeEnvContext
 from ray._private.services import ProcessInfo, start_ray_client_server
 from ray._private.tls_utils import add_port_to_grpc_server
 from ray._private.utils import detect_fate_sharing_support
-from ray._common.network_utils import build_address
+from ray._raylet import GcsClient
 from ray.cloudpickle.compat import pickle
 from ray.job_config import JobConfig
 from ray.util.client.common import (
@@ -40,6 +37,9 @@ from ray.util.client.common import (
 )
 from ray.util.client.server.dataservicer import _get_reconnecting_from_context
 
+# Import psutil after ray so the packaged version is used.
+import psutil
+
 logger = logging.getLogger(__name__)
 
 CHECK_PROCESS_INTERVAL_S = 30
@@ -860,7 +860,7 @@ def serve_proxier(
     ray_client_pb2_grpc.add_RayletDriverServicer_to_server(task_servicer, server)
     ray_client_pb2_grpc.add_RayletDataStreamerServicer_to_server(data_servicer, server)
     ray_client_pb2_grpc.add_RayletLogStreamerServicer_to_server(logs_servicer, server)
-    if host != "127.0.0.1" and host != "localhost":
+    if not is_localhost(host):
         add_port_to_grpc_server(server, f"127.0.0.1:{port}")
     add_port_to_grpc_server(server, f"{host}:{port}")
     server.start()
